﻿#pragma once


#include "core/pixel_format/pixel.h"

#include <unordered_map>

class renderer_texture_array;
struct stbtt_fontinfo;

struct character_info {
    float tex_u; // U position in atlas
    float tex_v; // V position in atlas
    float advance; // Advance width
    uint8_t tex_z; // Z position in texture array
    uint8_t width; // Character width
    uint8_t height; // Character height
    int8_t x_offset; // X offset from baseline
    int8_t y_offset; // Y offset from baseline
};

class aorii_text {
public:
    aorii_text(uint32_t in_texture_size = 2048);
    ~aorii_text();

    static bool init_freetype();
    static void destroy_freetype();

    bool initialize(const wchar_t* in_font_path, float in_font_pixel_size);
    bool get_or_create_character(wchar_t ch, character_info& out_info);
    // 上升距离
    [[nodiscard]] int32_t get_ascent() const { return ascent; }
    // 下降距离
    [[nodiscard]] int32_t get_descent() const { return descent; }
    // 行间距
    [[nodiscard]] int32_t get_line_gap() const { return line_gap; }
    // 纹理尺寸
    [[nodiscard]] uint32_t get_texture_size() const { return texture_size; }
    // 字体像素大小
    [[nodiscard]] float get_font_pixel_size() const { return font_pixel_size; }

    // 预缓存一组字符
    bool precache_characters(const std::wstring& characters);

    // 预缓存常用字符集
    bool precache_common_characters();

    // 获取缓存状态
    [[nodiscard]] size_t get_cached_character_count() const { return character_map.size(); }

    // 获取字体字符数量
    [[nodiscard]] uint32_t get_font_char_count() const;

    [[nodiscard]] renderer_texture_array* get_texture_array() const { return texture_array; }

    float get_kerning(wchar_t ch1, wchar_t ch2) const;
private:
    bool create_new_texture_atlas();
    character_info& add_character_to_atlas(int32_t glyph_index, const wchar_t ch);
    stbtt_fontinfo* font;

    renderer_texture_array* texture_array;
    const uint32_t texture_size;
    float font_pixel_size;
    uint32_t current_x;
    uint32_t current_y;
    uint32_t next_row_y = 0;  // 跟踪当前行的最大高度
    uint8_t current_texture_index;
    int32_t ascent, descent, line_gap;
    float scale;

    std::unordered_map<wchar_t, character_info> character_map;

    // 常用字符集定义
    static const std::wstring COMMON_ASCII;             // ASCII字符
    static const std::wstring COMMON_CHINESE;           // 常用汉字
    static const std::wstring COMMON_PUNCTUATION;       // 常用标点符号
    static const std::wstring COMMON_NUMBERS;           // 数字
};
